#include "event_queue.h"
#include "com/debug.h"

typedef struct {
	void (*handler_arg)(uint32_t);
	void (*handler_noarg)(void);
	uint32_t arg;
} QueuedTask;


/** Task queue */
static CircBuf *tq;

/** Event queue */
static CircBuf *eq;


void queues_init(size_t tq_size, size_t eq_size)
{
	tq = cbuf_create(tq_size, sizeof(QueuedTask));
	eq = cbuf_create(eq_size, sizeof(Event));
}


bool tq_post(void (*handler)(void))
{
	QueuedTask task;
	task.handler_noarg = handler;
	task.handler_arg = NULL;

	bool suc = cbuf_append(tq, &task);
	if (!suc) error("TQ overflow");
	return suc;
}


bool tq_post_arg(void (*handler)(uint32_t), uint32_t arg)
{
	QueuedTask task;
	task.handler_arg = handler;
	task.handler_noarg = NULL;
	task.arg = arg;

	bool suc = cbuf_append(tq, &task);
	if (!suc) error("TQ overflow");
	return suc;
}


bool eq_post(const Event *event)
{
	bool suc = cbuf_append(eq, event);
	if (!suc) {
		error("EQ overflow, evt %d", event->type);
	}
	return suc;
}


bool tq_poll_one(void)
{
	QueuedTask task;

	// serve all tasks
	bool suc = cbuf_pop(tq, &task);

	if (suc) {
		if (task.handler_arg) {
			task.handler_arg(task.arg);
		} else {
			task.handler_noarg();
		}
	}

	return suc;
}


void tq_poll(void)
{
	// serve all tasks
	while (tq_poll_one());
}


bool eq_take(Event *dest)
{
	bool suc = cbuf_pop(eq, dest);
	return suc;
}
