#include "buttons.h"
#include "hw_utils/all.h"

#include "utils/debounce.h"
#include "utils/timebase.h"
#include "bus/event_queue.h"

static void (*user_btn1_rising_cb)(void);
static void (*user_btn1_falling_cb)(void);
static void (*user_btn1_change_cb)(uint32_t state);

static debo_id_t btn1_id;


// --- callbacks ---

/** Rising callback */
static void btn1_rising_cb(void)
{
	if (user_btn1_rising_cb) tq_post(user_btn1_rising_cb);
	if (user_btn1_change_cb) tq_post_arg(user_btn1_change_cb, 1);
}


/** Falling callback */
static void btn1_falling_cb(void)
{
	if (user_btn1_falling_cb) tq_post(user_btn1_falling_cb);
	if (user_btn1_change_cb) tq_post_arg(user_btn1_change_cb, 0);
}

// ---


/** Initialize debouncer for buttons */
void buttons_init(void)
{
	debo_init_t debo_init = {
		.GPIOx = GPIOC,
		.pin = GPIO15,
		.debo_time = 80,
		.invert = true,
		.rising_cb = btn1_rising_cb,
		.falling_cb = btn1_falling_cb
	};

	btn1_id = debo_register_pin(&debo_init);
}


/** Check if button is pressed */
bool button_pressed(enum button_enum button)
{
	switch (button) {
		case BTN1: return debo_pin_state(btn1_id);
	}

	return false;
}

// --- set callback ---

/** Assign a rising callback */
void btn_set_press_callback(enum button_enum button, void (*cb)(void))
{
	switch (button) {
		case BTN1:
			user_btn1_rising_cb = cb;
	}
}


/** Assign a falling callback */
void btn_set_release_callback(enum button_enum button, void (*cb)(void))
{
	switch (button) {
		case BTN1:
			user_btn1_falling_cb = cb;
	}
}


/** Assign a change callback */
void btn_set_change_callback(enum button_enum button, void (*cb)(uint32_t))
{
	switch (button) {
		case BTN1:
			user_btn1_change_cb = cb;
	}
}
