#pragma once

/* Includes ------------------------------------------------------------------*/

#include "main.h"

/* Exported types ------------------------------------------------------------*/
/* Exported constants --------------------------------------------------------*/

// Short pin names
#define GPIO0                 ((uint16_t)0x0001)  /*!< Pin 0 selected    */
#define GPIO1                 ((uint16_t)0x0002)  /*!< Pin 1 selected    */
#define GPIO2                 ((uint16_t)0x0004)  /*!< Pin 2 selected    */
#define GPIO3                 ((uint16_t)0x0008)  /*!< Pin 3 selected    */
#define GPIO4                 ((uint16_t)0x0010)  /*!< Pin 4 selected    */
#define GPIO5                 ((uint16_t)0x0020)  /*!< Pin 5 selected    */
#define GPIO6                 ((uint16_t)0x0040)  /*!< Pin 6 selected    */
#define GPIO7                 ((uint16_t)0x0080)  /*!< Pin 7 selected    */
#define GPIO8                 ((uint16_t)0x0100)  /*!< Pin 8 selected    */
#define GPIO9                 ((uint16_t)0x0200)  /*!< Pin 9 selected    */
#define GPIO10                ((uint16_t)0x0400)  /*!< Pin 10 selected   */
#define GPIO11                ((uint16_t)0x0800)  /*!< Pin 11 selected   */
#define GPIO12                ((uint16_t)0x1000)  /*!< Pin 12 selected   */
#define GPIO13                ((uint16_t)0x2000)  /*!< Pin 13 selected   */
#define GPIO14                ((uint16_t)0x4000)  /*!< Pin 14 selected   */
#define GPIO15                ((uint16_t)0x8000)  /*!< Pin 15 selected   */
#define GPIO_All              ((uint16_t)0xFFFF)  /*!< All pins selected */

/* Exported macro ------------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */

/**
 * @brief Configure a GPIO output mode
 *
 * @param GPIOx      : port base
 * @param GPIO_Speed : output change speed
 * @param GPIO_OType : output type (push-pull, open-drain)
 * @param pins       : mask of affected pins
 */
void gpio_conf_output(GPIO_TypeDef* GPIOx, GPIOSpeed_TypeDef GPIO_Speed, GPIOOType_TypeDef GPIO_OType, uint16_t pins);


/**
 * @brief Configure a GPIO input mode
 *
 * @param GPIOx      : port base
 * @param GPIO_PuPd  : pull-up / pull-down mode
 * @param pins       : mask of affected pins
 */
void gpio_conf_input(GPIO_TypeDef* GPIOx, GPIOPuPd_TypeDef GPIO_PuPd, uint16_t pins);


/**
 * @brief Configure analog GPIO
 *
 * @param GPIOx      : port base
 * @param pins       : mask of affected pins
 */
void gpio_conf_analog(GPIO_TypeDef* GPIOx, uint16_t pins);


/**
 * @brief Configure the Alternate Function of GPIO
 *
 * @param GPIOx      : port base
 * @param GPIO_Speed : output change speed
 * @param GPIO_OType : output type (push-pull, open-drain)
 * @param af         : alternate function number (GPIO_AF_*)
 * @param pins       : mask of affected pins
 */
void gpio_conf_af(GPIO_TypeDef* GPIOx, GPIOSpeed_TypeDef GPIO_Speed, GPIOOType_TypeDef GPIO_OType, uint8_t af, uint16_t pins);


// --- specific ---

/**
 * @brief Reset the ESP module (set reset line state
 * @param nrst_assert true to pull reset down, false to release (boot esp)
 */
void esp_reset(bool nrst_assert);
