#include "usart.h"


/**
 * @brief Set USART baud rate.
 *
 * Mostly copied from SPL, because no function to set baud rate separately was provided.
 *
 * @param USARTx : USART peripheral
 * @param baud   : baud rate
 */
void USART_SetBaudrate(USART_TypeDef* USARTx, uint32_t baud)
{
	assert_param(IS_USART_BAUDRATE(baud));

	USART_Cmd(USARTx, DISABLE);

	// Following code copied from SPL (no separate function for baud rate provided)

	/*---------------------------- USART BRR Configuration -----------------------*/
	uint32_t divider = 0, apbclock = 0, tmpreg = 0;
	RCC_ClocksTypeDef RCC_ClocksStatus;

	RCC_GetClocksFreq(&RCC_ClocksStatus);

	if (USARTx == USART1) {
		apbclock = RCC_ClocksStatus.USART1CLK_Frequency;
	} else if (USARTx == USART2) {
		apbclock = RCC_ClocksStatus.USART2CLK_Frequency;
	} else if (USARTx == USART3) {
		apbclock = RCC_ClocksStatus.USART3CLK_Frequency;
	} else if (USARTx == UART4) {
		apbclock = RCC_ClocksStatus.UART4CLK_Frequency;
	} else {
		apbclock = RCC_ClocksStatus.UART5CLK_Frequency;
	}

	/* Determine the integer part */
	if ((USARTx->CR1 & USART_CR1_OVER8) != 0) {
		/* (divider * 10) computing in case Oversampling mode is 8 Samples */
		divider = (uint32_t)((2 * apbclock) / (baud));
		tmpreg  = (uint32_t)((2 * apbclock) % (baud));
	} else { /* if ((USARTx->CR1 & CR1_OVER8_Set) == 0) */
		/* (divider * 10) computing in case Oversampling mode is 16 Samples */
		divider = (uint32_t)((apbclock) / (baud));
		tmpreg  = (uint32_t)((apbclock) % (baud));
	}

	/* round the divider : if fractional part i greater than 0.5 increment divider */
	if (tmpreg >= (baud) / 2) {
		divider++;
	}

	/* Implement the divider in case Oversampling mode is 8 Samples */
	if ((USARTx->CR1 & USART_CR1_OVER8) != 0) {
		/* get the LSB of divider and shift it to the right by 1 bit */
		tmpreg = (divider & (uint16_t)0x000F) >> 1;
		/* update the divider value */
		divider = (divider & (uint16_t)0xFFF0) | tmpreg;
	}

	/* Write to USART BRR */
	USARTx->BRR = (uint16_t)divider;

	// -------------

	USART_Cmd(USARTx, ENABLE);
}

